import unittest
from shutil import copyfile
import pathlib
from sqlalchemy import create_engine
from sqlalchemy.event import listen
from SeismicFoldDbGis.FoldDbGis import FoldDbGis


def load_spatialite(dbapi_conn, connection_record):
    dbapi_conn.enable_load_extension(True)
    dbapi_conn.load_extension('/usr/lib/x86_64-linux-gnu/mod_spatialite.so')


class FoldDbGisTestCase(unittest.TestCase):
    def test(self):
        csv_file = 'tests/data/test-simple.fold.csv'
        empty_file = 'tests/data/empty.sqlite'
        db_file = 'tests/data/fold.sqlite'
        copyfile(empty_file, db_file)

        engine = create_engine('sqlite:///' + db_file, echo=False)
        listen(engine, 'connect', load_spatialite)

        fold = FoldDbGis(db_engine=engine, verbose=True)

        fold.create_table()

        fold.delete_table()

        fold.create_table()

        fold.load_from_csv(csv_file)

        fold.update_from_csv(csv_file)

        rem = pathlib.Path(db_file)
        rem.unlink(missing_ok=True)


if __name__ == '__main__':
    unittest.main()



import csv
from sqlalchemy.orm import sessionmaker
from SeismicFoldDbGis.entity.Bin import Bin


class FoldDbGis:

    def __init__(self, db_engine, commit_every=10000, verbose=False):
        self.__db_engine = db_engine
        self.__commit_every = commit_every
        self.__verbose = verbose

    def create_table(self):
        Bin.__table__.create(self.__db_engine)

    def delete_table(self):
        Bin.__table__.drop(self.__db_engine)

    def load_from_csv(self, filename: str):
        """
        load from CSV with columns ['Easting', 'Northing', 'Fold', 'Bin Number', 'Row', 'Column']
        """
        _session = self.__create_session()
        counter = 1
        verbose = 1
        with open(filename, 'r') as csv_file:
            csv_reader = csv.reader(csv_file, delimiter=',')
            next(csv_reader)
            for record in csv_reader:
                bin = FoldDbGis.__create_bin_from_csv_record(record)
                _session.add(bin)

                if self.__commit_every == counter:
                    if self.__verbose is True:
                        print("{:15,d}".format(verbose))
                    _session.commit()
                    counter = 1
                else:
                    counter += 1
                if self.__verbose is True:
                    verbose += 1

            if self.__verbose is True:
                print("{:15,d}".format(verbose))

            _session.commit()
        _session.close()

    def update_from_csv(self, filename: str):
        """
        update from CSV with columns ['Easting', 'Northing', 'Fold', 'Bin Number', 'Row', 'Column']
        """
        _session = self.__create_session()
        counter = 1
        verbose = 1
        with open(filename, 'r') as csv_file:
            csv_reader = csv.reader(csv_file, delimiter=',')
            next(csv_reader)
            for record in csv_reader:
                binn = int(record[3])
                b = _session.query(Bin).filter_by(binn=binn).first()

                if b is not None:
                    b.fold = b.fold + int(record[2])
                else:
                    b = FoldDbGis.__create_bin_from_csv_record(record)
                    _session.add(b)

                if self.__commit_every == counter:
                    if self.__verbose is True:
                        print("{:15,d}".format(verbose))
                    _session.commit()
                    counter = 1
                else:
                    counter += 1
                if self.__verbose is True:
                    verbose += 1

            if self.__verbose is True:
                print("{:15,d}".format(verbose))

            _session.commit()
        _session.close()

    @staticmethod
    def __create_bin_from_csv_record(record: list):
        b = Bin(
            binn=int(record[3]),
            fold=int(record[2]),
            geom="POINT (%.1f %.1f)" % (float(record[0]), float(record[1]))
        )

        return b

    def __create_session(self):
        sess_mkr = sessionmaker()
        sess_mkr.configure(bind=self.__db_engine)
        return sess_mkr()



"""
    SETUP
"""
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="SeismicFoldDbGis",
    version="0.0.4",
    author="Piotr Synowiec",
    author_email="psynowiec@gmail.com",
    description="Loads calculated fold from CSV file to GIS database",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mysiar/seismic-fold_db_gis-python-package",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



from sqlalchemy import Column, Integer
from geoalchemy2 import Geometry
from SeismicFoldDbGis.entity.Base import Base

TABLE_NAME_BIN = 'bins'


class Bin(Base):
    __tablename__ = TABLE_NAME_BIN
    id = Column(Integer, primary_key=True)
    binn = Column(Integer, index=True, unique=True)
    fold = Column(Integer)
    geom = Column(Geometry(geometry_type='POINT', management=True))

